@@ -50,7 +50,8 @@ static Display           *dpy;
 static Window             root_window, mag_window;
 static GQueue            *mag_windows_list;
 static Picture            off_screen_picture;
-static XserverRegion      tmp_region, new_region, old_region, exp_region;
+static XserverRegion      tmp_region = None, new_region = None,
+	                  old_region = None, exp_region = None;
 static DamageHandlerFunc  gmag_compositor_damage_notify_handler;
 
 
@@ -823,10 +824,6 @@ gmag_compositor_init (Magnifier *magnifier, Display *dpy_conn)
 
 		XCompositeRedirectSubwindows (dpy, root_window,
 					      CompositeRedirectAutomatic);
-		tmp_region = XFixesCreateRegion (dpy, 0, 0);
-		new_region = XFixesCreateRegion (dpy, 0, 0);
-		old_region = XFixesCreateRegion (dpy, 0, 0);
-		exp_region = XFixesCreateRegion (dpy, 0, 0);
 
 		format = XRenderFindVisualFormat (
 			dpy,
@@ -842,6 +839,12 @@ gmag_compositor_init (Magnifier *magnifier, Display *dpy_conn)
 						     gdkscr);
 		gmag_compositor_calculate_windows_clip ();
 		gmag_compositor_paint_screen ();
+
+		tmp_region = XFixesCreateRegion (dpy, 0, 0);
+		new_region = XFixesCreateRegion (dpy, 0, 0);
+		old_region = XFixesCreateRegion (dpy, 0, 0);
+		exp_region = XFixesCreateRegion (dpy, 0, 0);
+
 		g_message ("added event source to composite connection");
 	}
 }
