@@ -31,6 +31,8 @@
 #include "gmag-compositor.h"
 #endif /* HAVE_COMPOSIE */
 
+#include <string.h>
+
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/extensions/Xfixes.h>
@@ -242,6 +244,8 @@ gboolean
 gmag_gs_reset_overlay_at_idle (gpointer data)
 {
 #ifdef HAVE_COMPOSITE
+	if (!use_compositor)
+		return FALSE;
 	if (!g_getenv ("MAGNIFIER_IGNORE_COMPOSITE")) {
 		if (data) {
 			Magnifier *magnifier = MAGNIFIER (data);
@@ -351,8 +355,6 @@ gmag_gs_magnifier_realize (GtkWidget *widget)
 	/* TODO: make sure this works/is reset if the DISPLAY 
 	 * (as well as the SCREEN) changes.
 	 */
-
-	XSetErrorHandler (gmag_gs_x_error_handler);
 }
 
 void
@@ -443,8 +445,9 @@ gmag_gs_use_damage ()
 	gint major, event, error;
 	if (XQueryExtension (
 		    dpy, "DAMAGE", &major, &damage_event_base, &error) &&
-	    !g_getenv ("MAGNIFIER_IGNORE_DAMAGE"))
+	    !g_getenv ("MAGNIFIER_IGNORE_DAMAGE")) {
 		return TRUE;
+	}
 	return FALSE;
 }
 
@@ -458,13 +461,21 @@ gmag_gs_use_damage ()
  * others.
  */
 static gboolean
-gmag_gs_use_compositor ()
+gmag_gs_use_compositor (Magnifier *magnifier)
 {
 	if (!gmag_gs_use_damage ()) {
 		return FALSE;
 	}
 #ifdef HAVE_COMPOSITE
 	gint major, event, error;
+
+	/* if the source and target display are different we don't need the
+	 * composite extension. This is the case when using the dummy driver or
+	 * a dual head configuration */
+	if (strcmp (magnifier->source_display_name,
+		    magnifier->target_display_name) != 0)
+		return FALSE;
+
 	if (XQueryExtension (dpy, "Composite", &major, &event, &error) &&
 	    !g_getenv ("MAGNIFIER_IGNORE_COMPOSITE"))
 		return TRUE;
@@ -480,6 +491,8 @@ gmag_gs_client_init (Magnifier *magnifier)
 	GIOChannel               *ioc;
 	gint                      fd;
 
+	XSetErrorHandler (gmag_gs_x_error_handler);
+
 	if (dpy) {
 		/* remove the old watch */
 		if (dpy_gsource) 
@@ -510,7 +523,7 @@ gmag_gs_client_init (Magnifier *magnifier)
 	g_io_channel_unref (ioc); 
 
 	use_damage = gmag_gs_use_damage ();
-	use_compositor = gmag_gs_use_compositor ();
+	use_compositor = gmag_gs_use_compositor (magnifier);
 
 	fixes_event_base = gmag_cursor_init (magnifier, dpy);
 
@@ -519,11 +532,14 @@ gmag_gs_client_init (Magnifier *magnifier)
 	 * damage notify handler, more suitable to the composite task. I think
 	 * that this leave the logic more simple.
 	 */
-	if (!use_compositor)
+	if (!use_compositor) {
 		gmag_damage_init (magnifier, dpy);
+		g_setenv ("MAGNIFIER_IGNORE_COMPOSITE", "1", FALSE);
+	}
 
 #ifdef HAVE_COMPOSITE
-	gmag_compositor_init (magnifier, dpy);
+	if (use_compositor)
+		gmag_compositor_init (magnifier, dpy);
 #endif /* HAVE_COMPOSITE */
 
 	XFlush (dpy);
